/*
 *   senator - ASF video stream server
 *   Copyright (C) 2002  anonymous author
 *
 *
 *  senator is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  Senator is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *	
 *  You should have received a copy of the GNU General Public License
 *  along with senator; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 */
    
#include <stdio.h>
#include "CConfigFile.h"
#include "CLog.h"

extern CLog	*logger;

#define CAN_IGNORE_LINE(line) \
  ((line[0] == '#')  || (line[0] == ' ') || \
  (line[0] == '\r') || (line[0] == '\n'))
     
#define GENERIC_BUF_LEN 255

CConfigFile::CConfigFile(const char *fileName){
	this->maxConnections	= 12;
	this->portNumber	= 4444;
	this->rootDirectory	= "/home/";
	this->identifyAs	= "Cougarama / senator server";
	this->HTTPStreamProtocol= true;

	this->_fileName = (std::string)fileName;
	
	this->_data	= 0;
	this->_length	= 0;
}

std::string CConfigFile::scanToken(const char * line){
	
    char option[GENERIC_BUF_LEN];
    char value[GENERIC_BUF_LEN];
	
	memset(option,0,GENERIC_BUF_LEN*sizeof(char));
	memset(value,0,GENERIC_BUF_LEN*sizeof(char));

    if (sscanf(line,"%s %s",option, value)) {
		return option;
    }

	return "";
}

bool CConfigFile::_getBool(const char *line){
    char option[GENERIC_BUF_LEN];
    char value[GENERIC_BUF_LEN];
	
    memset(option,0,GENERIC_BUF_LEN*sizeof(char));
    memset(value,0,GENERIC_BUF_LEN*sizeof(char));

// true,1,yes,yeppers,go!
// false,0,no,nope,bleuh

    if (sscanf(line,"%s %s",option, value)) {
		if (memcmp(value,"true",4) == 0)
			return true;
		if (memcmp(value,"1",1) == 0)
			return true;
		if (memcmp(value,"yes",3) == 0)
			return true;
		if (memcmp(value,"yeppers",7) == 0)
			return true;
		if (memcmp(value,"go!",3) == 0)
			return true;

		if (memcmp(value,"false",5) == 0)
			return false;
		if (memcmp(value,"0",1) == 0)
			return false;
		if (memcmp(value,"no",2) == 0)
			return false;
		if (memcmp(value,"nope",4) == 0)
			return false;
		if (memcmp(value,"bleuh",5) == 0)
			return false;
	}
	return false;
}

int CConfigFile::_getInt(const char *line){
    char option[GENERIC_BUF_LEN];
    char value[GENERIC_BUF_LEN];
	
    memset(option,0,GENERIC_BUF_LEN*sizeof(char));
    memset(value,0,GENERIC_BUF_LEN*sizeof(char));

    if (sscanf(line,"%s %s",option, value)) {
		return atoi(value);
    }
    return 0;
}

std::string CConfigFile::_getString(const char *line){
    char option[GENERIC_BUF_LEN];
    char value[GENERIC_BUF_LEN];
	
    memset(option,0,GENERIC_BUF_LEN*sizeof(char));
    memset(value,0,GENERIC_BUF_LEN*sizeof(char));

    if (sscanf(line,"%s %s",option, value)) {
		return (std::string) value;
    }
    return "";
}


bool CConfigFile::parse(){
	logger->logText("config file : " + _fileName);
	FILE *fd = fopen(_fileName.c_str(),"r");
	
	if (fd) {
		char line[GENERIC_BUF_LEN];
		memset(line,0,GENERIC_BUF_LEN*sizeof(char));
					    
		while(fgets(line,GENERIC_BUF_LEN,fd)){
			if (CAN_IGNORE_LINE(line)){
				continue;
			}

			if (scanToken(line) == "HTTPStreamProtocol")
				this->HTTPStreamProtocol = _getBool(line);
			
			if (scanToken(line) == "portNumber")
				this->portNumber = _getInt(line);

			if (scanToken(line) == "maxConnections")
				this->maxConnections = _getInt(line);

			if (scanToken(line) == "rootDirectory")
				this->rootDirectory = _getString(line);

			if (scanToken(line) == "identifyAs")
				this->identifyAs = _getString(line);

			if (scanToken(line) == "statusPageName")
				this->statusPageName = _getString(line);
		}
		fclose(fd);
		return true;
	} 

	return false;
}
